# PROGRAM 13.1   TVVAR
tvvarOmp <- function(y, trend.order, tau2.ini = NULL, delta, plot = TRUE, ...)    
{
  n <- length(y)             # length of data
  m <- trend.order           # trend order
# tau2.ini                   # initial variance of systen noise
# delta                      # delta for computing variance of system noise
  iopt <- 1                  # search method
  if (is.null(tau2.ini)) {
    iopt  <- 0
    tau2.ini <- 0
    delta <- 0
  }

  n2 <- n / 2
  
  z <- .Fortran(C_tvvaromp,
                as.double(y),    
                as.integer(n),
                as.integer(m),
                as.double(tau2.ini),
                as.integer(iopt),
                as.double(delta),
                tvvar = double(n2),
                normdat = double(n),
                y1 = double(n2),
                n1 = integer(1),
                trend = double(3*n2),
                noise = double(n2),
                taumax = double(1),
                sigm2m = double(1),
                ffmax = double(1),
                aic = double(1))

  trend <- array(z$trend, dim=c(z$n1,3))

  out <- list(tvv = z$tvvar, nordata = z$normdat, sm = z$y1, trend = trend,
              noise = z$noise, tau2 = z$taumax, sigma2 = z$sigm2m,
              llkhood = z$ffmax, aic = z$aic,tsname = deparse(substitute(y)))
  class(out) <- "tvvar"

  if (plot)
    plot.tvvar(out, ...)

  return(out)
}


# PROGRAM 13.2  TVAR
tvarOmp <- function (y, trend.order = 2, ar.order = 2, span, outlier = NULL,
                     tau2.ini = NULL, delta, plot = TRUE)
{
# y                    # original data
  n <- length(y)       # data length
# ar.order             # AR order
# trend.order          # Trend order
  if (trend.order != 1 && trend.order != 2)
    stop("'trend.order' is 1 or 2")
# span                 # local stationary span
# outlier              # position of i-th outlier
  if (is.null(outlier)) {
    nout <- 0          # number of outliers
    outlier <- 0
  } else {
    nout <- length(outlier)
  }
# tau2.ini             # initial variance of systen noise
# delta                # delta for computing variance of system noise
  iopt <- 1            # search method
  if (is.null(tau2.ini)) {
    iopt  <- 0
    tau2.ini <- 0
    delta <- 0
  }
  
  nn <- n/span
  k <-  ar.order * nn

  z <- .Fortran(C_tvaromp,
                as.double(y),
                as.integer(n),
                as.integer(ar.order),
                as.integer(trend.order),
                as.integer(span),
                as.integer(iopt),
                as.integer(nout),
                as.integer(outlier),
                as.double(tau2.ini),
                as.double(delta),
                tau2 = double(1),
                sigma2 = double(1),
                lkhood = double(1),
                aic = double(1),
                arcoef = double(k),
                parcor = double(k))

  arcoef <- array(z$arcoef, dim = c(ar.order, nn))
  parcor <- array(z$parcor, dim = c(ar.order, nn))

  out <- list(arcoef = arcoef, sigma2 = z$sigma2, tau2 = z$tau2,    
              llkhood = z$lkhood, aic = z$aic, parcor = parcor)
  class(out) <- "tvvar"

  if (plot)
    plot.parcor(parcor, span)

  return(out)
}

plot.parcor <- function(parcor, span)
{
  ar.order <- dim(parcor)[1]
  n <- dim(parcor)[2]

  x <- span
  for (i in 2:n)
    x <- c(x, i*span)

  if (ar.order > 3)
    nc <- 3
  old.par <- par(no.readonly = TRUE)
  par(mfrow = c(nc, 1))

  for (i in 1:ar.order ) {
    if ((i%%3 == 1) & i > 1)
      par(ask = TRUE)
      plot(x, parcor[i, ], type = "l", xlab = "", ylim = c(-1.0, 1.0),
           ylab = paste("parcor[", i, ", ]"), xaxs = "i", yaxs = "i")
  }

  par(old.par)
}
