\name{tvarOmp}
\alias{tvarOmp}

\title{Time Varying Coefficients AR Model}
\description{
  Estimate time varying coefficients AR model.
}
\usage{
tvarOmp(y, trend.order = 2, ar.order = 2, span, outlier = NULL, tau2.ini = NULL,
        delta, plot = TRUE)
}
\arguments{
  \item{y}{a univariate time series.}
  \item{trend.order}{trend order (1 or 2).}
  \item{ar.order}{AR order.}
  \item{span}{local stationary span.}
  \item{outlier}{positions of outliers.}
  \item{tau2.ini}{initial estimate of variance of the system noise \eqn{\tau^2}.
    If \code{tau2.ini} = \code{NULL}, the most suitable value is chosen in
    \eqn{\tau^2 = 2^{-k}}.}
  \item{delta}{search width.} 
  \item{plot}{logical. If \code{TRUE} (default), PARCOR is plotted.}
}
\details{
  The time-varying coefficients AR model is given by

    \deqn{y_t = a_{1,t}y_{t-1} + \ldots + a_{p,t}y_{t-p} + u_t}{%
          y(t) = a(1,t)y(t-1) + ... + a(p,t)y(t-p) + u(t)}

    where \eqn{a_{i,t}}{a(i,t)} is \eqn{i}-lag AR coefficient at time \eqn{t}
    and \eqn{u_t}{u(t)} is a zero mean white noise.
}
\value{
  \item{arcoef}{time varying AR coefficients.}
  \item{sigma2}{variance of the observational noise \eqn{\sigma^2}.}
  \item{tau2}{variance of the system noise \eqn{\tau^2}.}
  \item{llkhood}{log-likelihood of the model.}
  \item{aic}{AIC.}
  \item{parcor}{PARCOR.}
}
\references{
  Kitagawa, G. (2020)
  \emph{Introduction to Time Series Modeling with Applications in R}.
  Chapman & Hall/CRC.

  Kitagawa, G. and Gersch, W. (1996)
  \emph{Smoothness Priors Analysis of Time Series}.
  Lecture Notes in Statistics, No.116, Springer-Verlag.

  Kitagawa, G. and Gersch, W. (1985)
  \emph{A smoothness priors time varying AR coefficient modeling of
  nonstationary time series}. IEEE trans. on Automatic Control, AC-30, 48-56.
}
\examples{
# seismic data
data(MYE1F)
system.time(z1 <- tvarOmp(MYE1F, trend.order = 2, ar.order = 8, span = 20,
            outlier = c(630, 1026), tau2.ini = 6.6e-06, delta = 1.0e-06))
z1

# tvspc() is the function in the "TSSS" package.
spec <- tvspc(z1$arcoef, z1$sigma2)
plot(spec)


## time varying test data : tvarData
ar <- list()
ar[[1]]<- c(0.9*sqrt(3), -0.81)
ar[[2]] <- c(0.7, 0.1)
ar[[3]] <- c(0.05, 0.2)
ar[[4]] <- c(-0.9, -0.81)
tvarData <- NULL
nd <- 500000
for (i in 1:4) {
  set.seed(100+i)
  tvarData <- c(tvarData, arima.sim(list(ar[[i]], ma=0), nd, sd=1))
}

system.time(z2 <- tvarOmp(tvarData, trend.order = 2, ar.order = 4, span = nd/10,
            tau2.ini = 6.6e-06, delta = 1.0e-06, plot = FALSE))
z2
}

\keyword{ts}
